@@ -4,7 +4,7 @@
 // Author:
 //   Aaron Bockover <abockover@novell.com>
 //
-// Copyright (C) 2007 Novell, Inc.
+// Copyright (C) 2007-2008 Novell, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -38,13 +38,22 @@ using Timer=Hyena.Timer;
 
 using Banshee.ServiceStack;
 using Banshee.Sources;
+using Banshee.Collection;
 using Banshee.Collection.Database;
 using Banshee.Streaming;
 
+// MIGRATION NOTE: Return true if the step should allow the driver to continue
+//                 Return false if the step should terminate driver
+
 namespace Banshee.Database
 {
     public class BansheeDbFormatMigrator
     {
+        // NOTE: Whenever there is a change in ANY of the database schema,
+        //       this version MUST be incremented and a migration method
+        //       MUST be supplied to match the new version number
+        protected const int CURRENT_VERSION = 11;
+        protected const int CURRENT_METADATA_VERSION = 1;
         
 #region Migration Driver
         
@@ -56,12 +65,7 @@ namespace Banshee.Database
 
         public event EventHandler Started;
         public event EventHandler Finished;
-        
-        // NOTE: Whenever there is a change in ANY of the database schema,
-        //       this version MUST be incremented and a migration method
-        //       MUST be supplied to match the new version number
-        protected const int CURRENT_VERSION = 1;
-        
+                
         protected class DatabaseVersionAttribute : Attribute 
         {
             private int version;
@@ -123,48 +127,66 @@ namespace Banshee.Database
             }
         }
         
-        public void Migrate()
+        public void Migrate ()
         {
             try {
-                Execute("BEGIN");
-                InnerMigrate();
-                Execute("COMMIT");
-            } catch(Exception e) {
-                Console.WriteLine("Rolling back transaction");
-                Console.WriteLine(e);
-                Execute("ROLLBACK");
+                
+                if (DatabaseVersion < CURRENT_VERSION) {
+                    Execute ("BEGIN");
+                    InnerMigrate ();
+                    Execute ("COMMIT");
+                } else {
+                    Log.DebugFormat ("Database version {0} is up to date", DatabaseVersion);
+                }
+                
+                // Trigger metadata refreshes if necessary
+                int metadata_version = connection.Query<int> ("SELECT Value FROM CoreConfiguration WHERE Key = 'MetadataVersion'");
+                if (DatabaseVersion == CURRENT_VERSION && metadata_version < CURRENT_METADATA_VERSION) {
+                    ServiceManager.ServiceStarted += OnServiceStarted;
+                }
+            } catch (Exception) {
+                Log.Warning ("Rolling back database migration");
+                Execute ("ROLLBACK");
+                throw;
             }
 
             OnFinished ();
         }
         
-        private void InnerMigrate()
-        {   
-            MethodInfo [] methods = GetType().GetMethods(BindingFlags.Instance | BindingFlags.NonPublic);
+        private void InnerMigrate ()
+        {
+            MethodInfo [] methods = GetType ().GetMethods (BindingFlags.Instance | BindingFlags.NonPublic);
             bool terminate = false;
             bool ran_migration_step = false;
             
-            for(int i = DatabaseVersion + 1; i <= CURRENT_VERSION; i++) {
-                foreach(MethodInfo method in methods) {
-                    foreach(Attribute attr in method.GetCustomAttributes(false)) {
-                        if(attr is DatabaseVersionAttribute && ((DatabaseVersionAttribute)attr).Version == i) {
-                            if (!ran_migration_step) {
-                                ran_migration_step = true;
-                                OnStarted ();
-                            }
-
-                            if(!(bool)method.Invoke(this, null)) {
-                                terminate = true;
-                            }
-                            break;
+            Log.DebugFormat ("Migrating from database version {0} to {1}", DatabaseVersion, CURRENT_VERSION);
+            for (int i = DatabaseVersion + 1; i <= CURRENT_VERSION; i++) {
+                foreach (MethodInfo method in methods) {
+                    foreach (DatabaseVersionAttribute attr in method.GetCustomAttributes (
+                        typeof (DatabaseVersionAttribute), false)) {
+                        if (attr.Version != i) {
+                            continue;
                         }
+                        
+                        if (!ran_migration_step) {
+                            ran_migration_step = true;
+                            OnStarted ();
+                        }
+
+                        if (!(bool)method.Invoke (this, null)) {
+                            terminate = true;
+                        }
+                        
+                        break;
                     }
                 }
                 
-                if(terminate) {
+                if (terminate) {
                     break;
                 }
             }
+            
+            Execute (String.Format ("UPDATE CoreConfiguration SET Value = {0} WHERE Key = 'DatabaseVersion'", CURRENT_VERSION));
         }
         
         protected bool TableExists(string tableName)
@@ -179,30 +201,23 @@ namespace Banshee.Database
             
         protected int DatabaseVersion {
             get {
-                if(!TableExists("CoreConfiguration")) {
+                if (!TableExists("CoreConfiguration")) {
                     return 0;
                 }
                 
-                string select_query = @"
-                    SELECT Value 
-                        FROM CoreConfiguration
-                        WHERE Key = 'DatabaseVersion'
-                ";
-                
-                return Convert.ToInt32 (connection.Query<int> (select_query));
+                return connection.Query<int> ("SELECT Value FROM CoreConfiguration WHERE Key = 'DatabaseVersion'");
             }
         }
         
 #endregion
         
-#region Migration Step Implementations
 #pragma warning disable 0169
-        // NOTE: Return true if the step should allow the driver to continue
-        //       Return false if the step should terminate driver
-        
+
+#region Version 1
+                                
         [DatabaseVersion (1)]
         private bool Migrate_1 ()
-        {   
+        {
             if (TableExists("Tracks")) {
                 InitializeFreshDatabase ();
                 
@@ -228,11 +243,175 @@ namespace Banshee.Database
                 InitializeFreshDatabase ();
                 return false;
             }
-        }   
+        }
         
+#endregion
+
+#region Version 2
+        
+        [DatabaseVersion (2)]
+        private bool Migrate_2 ()
+        {
+            Execute (String.Format ("ALTER TABLE CoreTracks ADD COLUMN Attributes INTEGER  DEFAULT {0}",
+                (int)TrackMediaAttributes.Default));
+            return true;
+        }
+
+#endregion
+
+#region Version 3
+
+        [DatabaseVersion (3)]
+        private bool Migrate_3 ()
+        {
+            Execute ("ALTER TABLE CorePlaylists ADD COLUMN PrimarySourceID INTEGER");
+            Execute ("UPDATE CorePlaylists SET PrimarySourceID = 1");
+
+            Execute ("ALTER TABLE CoreSmartPlaylists ADD COLUMN PrimarySourceID INTEGER");
+            Execute ("UPDATE CoreSmartPlaylists SET PrimarySourceID = 1");
+            return true;
+        }
+
+#endregion
+
+#region Version 4
+
+        [DatabaseVersion (4)]
+        private bool Migrate_4 ()
+        {
+            Execute ("ALTER TABLE CoreTracks ADD COLUMN LastSkippedStamp INTEGER");
+            return true;
+        }
+
+#endregion
+
+#region Version 5
+        
+        [DatabaseVersion (5)]
+        private bool Migrate_5 ()
+        {
+            Execute ("ALTER TABLE CoreTracks ADD COLUMN TitleLowered TEXT");
+            Execute ("ALTER TABLE CoreArtists ADD COLUMN NameLowered TEXT");
+            Execute ("ALTER TABLE CoreAlbums ADD COLUMN TitleLowered TEXT");
+
+            // Set default so sorting isn't whack while we regenerate
+            Execute ("UPDATE CoreTracks SET TitleLowered = lower(Title)");
+            Execute ("UPDATE CoreArtists SET NameLowered = lower(Name)");
+            Execute ("UPDATE CoreAlbums SET TitleLowered = lower(Title)");
+
+            // Drop old indexes
+            Execute ("DROP INDEX IF EXISTS CoreTracksPrimarySourceIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksArtistIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksAlbumIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksRatingIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksLastPlayedStampIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksDateAddedStampIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksPlayCountIndex");
+            Execute ("DROP INDEX IF EXISTS CoreTracksTitleIndex");
+            Execute ("DROP INDEX IF EXISTS CoreAlbumsIndex");
+            Execute ("DROP INDEX IF EXISTS CoreAlbumsArtistID");
+            Execute ("DROP INDEX IF EXISTS CoreArtistsIndex");
+            Execute ("DROP INDEX IF EXISTS CorePlaylistEntriesIndex");
+            Execute ("DROP INDEX IF EXISTS CorePlaylistTrackIDIndex");
+            Execute ("DROP INDEX IF EXISTS CoreSmartPlaylistEntriesPlaylistIndex");
+            Execute ("DROP INDEX IF EXISTS CoreSmartPlaylistEntriesTrackIndex");
+
+            // Create new indexes
+            Execute ("CREATE INDEX IF NOT EXISTS CoreTracksIndex ON CoreTracks(ArtistID, AlbumID, PrimarySourceID, Disc, TrackNumber, Uri)");
+            Execute ("CREATE INDEX IF NOT EXISTS CoreArtistsIndex ON CoreArtists(NameLowered)");
+            Execute ("CREATE INDEX IF NOT EXISTS CoreAlbumsIndex       ON CoreAlbums(ArtistID, TitleLowered)");
+            Execute ("CREATE INDEX IF NOT EXISTS CoreSmartPlaylistEntriesIndex ON CoreSmartPlaylistEntries(SmartPlaylistID, TrackID)");
+            Execute ("CREATE INDEX IF NOT EXISTS CorePlaylistEntriesIndex ON CorePlaylistEntries(PlaylistID, TrackID)");
+            
+            return true;
+        }
+
+#endregion
+
+#region Version 6
+
+        [DatabaseVersion (6)]
+        private bool Migrate_6 ()
+        {
+            Execute ("INSERT INTO CoreConfiguration VALUES (null, 'MetadataVersion', 0)");
+            return true;
+        }
+        
+#endregion
+
+#region Version 7
+
+        [DatabaseVersion (7)]
+        private bool Migrate_7 ()
+        {
+            Execute ("UPDATE CorePrimarySources SET StringID = 'MusicLibrarySource-Library' WHERE StringID = 'Library'");
+            Execute ("UPDATE CorePrimarySources SET StringID = 'VideoLibrarySource-VideoLibrary' WHERE StringID = 'VideoLibrary'");
+            Execute ("UPDATE CorePrimarySources SET StringID = 'PodcastSource-podcasting' WHERE StringID = 'podcasting'");
+            Execute ("DELETE FROM CoreCache; DELETE FROM CoreCacheModels");
+            return true;
+        }
+        
+#endregion
+
+#region Version 8
+
+        [DatabaseVersion (8)]
+        private bool Migrate_8 ()
+        {
+            Execute ("ALTER TABLE CorePrimarySources ADD COLUMN CachedCount INTEGER");
+            Execute ("ALTER TABLE CorePlaylists ADD COLUMN CachedCount INTEGER");
+            Execute ("ALTER TABLE CoreSmartPlaylists ADD COLUMN CachedCount INTEGER");
+
+            // This once, we need to reload all the sources at start up. Then never again, woo!
+            Application.ClientStarted += ReloadAllSources;
+            return true;
+        }
+        
+#endregion
+
+#region Version 9
+
+        [DatabaseVersion (9)]
+        private bool Migrate_9 ()
+        {
+            Execute (String.Format ("ALTER TABLE CoreTracks ADD COLUMN LastStreamError INTEGER DEFAULT {0}",
+                (int)StreamPlaybackError.None));
+            return true;
+        }
+
+#endregion
+
+#region Version 10
+
+        [DatabaseVersion (10)]
+        private bool Migrate_10 ()
+        {
+            // Clear these out for people who ran the pre-alpha podcast plugin
+            Execute ("DROP TABLE IF EXISTS PodcastEnclosures");
+            Execute ("DROP TABLE IF EXISTS PodcastItems");
+            Execute ("DROP TABLE IF EXISTS PodcastSyndications");
+            Execute ("ALTER TABLE CoreTracks ADD COLUMN ExternalID INTEGER");
+            return true;
+        }
+        
+#endregion
+
+#region Version 11
+        
+        [DatabaseVersion (11)]
+        private bool Migrate_11 ()
+        {
+            Execute("CREATE INDEX CoreTracksExternalIDIndex ON CoreTracks(PrimarySourceID, ExternalID)");
+            return true;
+        }
+        
+#endregion
+
 #pragma warning restore 0169
         
-        private void InitializeFreshDatabase()
+#region Fresh database setup
+        
+        private void InitializeFreshDatabase ()
         {
             Execute("DROP TABLE IF EXISTS CoreConfiguration");
             Execute("DROP TABLE IF EXISTS CoreTracks");
@@ -253,31 +432,29 @@ namespace Banshee.Database
                     Value               TEXT
                 )
             ");
-            
-            Execute (String.Format (
-                "INSERT INTO CoreConfiguration VALUES (null, 'DatabaseVersion', '{0}')",
-                CURRENT_VERSION
-            ));
-            
+            Execute (String.Format ("INSERT INTO CoreConfiguration VALUES (null, 'DatabaseVersion', {0})", CURRENT_VERSION));
+            Execute ("INSERT INTO CoreConfiguration VALUES (null, 'MetadataVersion', 0)");
             
             Execute(@"
                 CREATE TABLE CorePrimarySources (
                     PrimarySourceID     INTEGER PRIMARY KEY,
-                    StringID            TEXT UNIQUE
+                    StringID            TEXT UNIQUE,
+                    CachedCount         INTEGER
                 )
             ");
-            Execute ("INSERT INTO CorePrimarySources (StringID) VALUES ('Library')");
+            Execute ("INSERT INTO CorePrimarySources (StringID) VALUES ('MusicLibrarySource-Library')");
 
             // TODO add these:
             // Others to consider:
             // AlbumArtist (TPE2) (in CoreAlbums?)
-            Execute(@"
+            Execute(String.Format (@"
                 CREATE TABLE CoreTracks (
                     PrimarySourceID     INTEGER NOT NULL,
                     TrackID             INTEGER PRIMARY KEY,
                     ArtistID            INTEGER,
                     AlbumID             INTEGER,
                     TagSetID            INTEGER,
+                    ExternalID          INTEGER,
                     
                     MusicBrainzID       TEXT,
 
@@ -285,8 +462,11 @@ namespace Banshee.Database
                     UriType             INTEGER,
                     MimeType            TEXT,
                     FileSize            INTEGER,
+                    Attributes          INTEGER DEFAULT {0},
+                    LastStreamError     INTEGER DEFAULT {1},
                     
                     Title               TEXT,
+                    TitleLowered        TEXT,
                     TrackNumber         INTEGER,
                     TrackCount          INTEGER,
                     Disc                INTEGER,
@@ -302,21 +482,14 @@ namespace Banshee.Database
                     PlayCount           INTEGER,
                     SkipCount           INTEGER,
                     LastPlayedStamp     INTEGER,
+                    LastSkippedStamp    INTEGER,
                     DateAddedStamp      INTEGER,
                     DateUpdatedStamp    INTEGER
                 )
-            ");
-            Execute("CREATE INDEX CoreTracksPrimarySourceIndex ON CoreTracks(PrimarySourceID)");
+            ", (int)TrackMediaAttributes.Default, (int)StreamPlaybackError.None));
+            Execute("CREATE INDEX CoreTracksPrimarySourceIndex ON CoreTracks(ArtistID, AlbumID, PrimarySourceID, Disc, TrackNumber, Uri)");
             Execute("CREATE INDEX CoreTracksAggregatesIndex ON CoreTracks(FileSize, Duration)");
-            Execute("CREATE INDEX CoreTracksArtistIndex ON CoreTracks(ArtistID)");
-            Execute("CREATE INDEX CoreTracksAlbumIndex  ON CoreTracks(AlbumID)");
-            Execute("CREATE INDEX CoreTracksRatingIndex ON CoreTracks(Rating)");
-            Execute("CREATE INDEX CoreTracksLastPlayedStampIndex ON CoreTracks(LastPlayedStamp)");
-            Execute("CREATE INDEX CoreTracksDateAddedStampIndex ON CoreTracks(DateAddedStamp)");
-            Execute("CREATE INDEX CoreTracksPlayCountIndex ON CoreTracks(PlayCount)");
-            Execute("CREATE INDEX CoreTracksDiscIndex ON CoreTracks(Disc)");
-            Execute("CREATE INDEX CoreTracksTrackNumberIndex ON CoreTracks(TrackNumber)");
-            Execute("CREATE INDEX CoreTracksTitleeIndex ON CoreTracks(Title)");
+            Execute("CREATE INDEX CoreTracksExternalIDIndex ON CoreTracks(PrimarySourceID, ExternalID)");
             
             Execute(@"
                 CREATE TABLE CoreAlbums (
@@ -327,6 +500,7 @@ namespace Banshee.Database
                     MusicBrainzID       TEXT,
 
                     Title               TEXT,
+                    TitleLowered        TEXT,
 
                     ReleaseDate         INTEGER,
                     Duration            INTEGER,
@@ -335,8 +509,7 @@ namespace Banshee.Database
                     Rating              INTEGER
                 )
             ");
-            Execute("CREATE INDEX CoreAlbumsIndex       ON CoreAlbums(Title)");
-            Execute("CREATE INDEX CoreAlbumsArtistID    ON CoreAlbums(ArtistID)");
+            Execute("CREATE INDEX CoreAlbumsIndex       ON CoreAlbums(ArtistID, TitleLowered)");
 
             Execute(@"
                 CREATE TABLE CoreArtists (
@@ -344,18 +517,21 @@ namespace Banshee.Database
                     TagSetID            INTEGER,
                     MusicBrainzID       TEXT,
                     Name                TEXT,
+                    NameLowered         TEXT,
                     Rating              INTEGER
                 )
             ");
-            Execute("CREATE INDEX CoreArtistsIndex      ON CoreArtists(Name)");
+            Execute("CREATE INDEX CoreArtistsIndex ON CoreArtists(NameLowered)");
             
             Execute(@"
                 CREATE TABLE CorePlaylists (
+                    PrimarySourceID     INTEGER,
                     PlaylistID          INTEGER PRIMARY KEY,
                     Name                TEXT,
                     SortColumn          INTEGER NOT NULL DEFAULT -1,
                     SortType            INTEGER NOT NULL DEFAULT 0,
-                    Special             INTEGER NOT NULL DEFAULT 0
+                    Special             INTEGER NOT NULL DEFAULT 0,
+                    CachedCount         INTEGER
                 )
             ");
             
@@ -367,17 +543,18 @@ namespace Banshee.Database
                     ViewOrder           INTEGER NOT NULL DEFAULT 0
                 )
             ");
-            Execute("CREATE INDEX CorePlaylistEntriesIndex ON CorePlaylistEntries(PlaylistID)");
-            Execute("CREATE INDEX CorePlaylistTrackIDIndex ON CorePlaylistEntries(TrackID)");
+            Execute("CREATE INDEX CorePlaylistEntriesIndex ON CorePlaylistEntries(PlaylistID, TrackID)");
             
             Execute(@"
                 CREATE TABLE CoreSmartPlaylists (
+                    PrimarySourceID     INTEGER,
                     SmartPlaylistID     INTEGER PRIMARY KEY,
                     Name                TEXT NOT NULL,
                     Condition           TEXT,
                     OrderBy             TEXT,
                     LimitNumber         TEXT,
-                    LimitCriterion      TEXT
+                    LimitCriterion      TEXT,
+                    CachedCount         INTEGER
                 )
             ");
                 
@@ -388,8 +565,7 @@ namespace Banshee.Database
                     TrackID             INTEGER NOT NULL
                 )
             ");
-            Execute("CREATE INDEX CoreSmartPlaylistEntriesPlaylistIndex ON CoreSmartPlaylistEntries(SmartPlaylistID)");
-            Execute("CREATE INDEX CoreSmartPlaylistEntriesTrackIndex ON CoreSmartPlaylistEntries(TrackID)");
+            Execute("CREATE INDEX CoreSmartPlaylistEntriesPlaylistIndex ON CoreSmartPlaylistEntries(SmartPlaylistID, TrackID)");
 
             Execute(@"
                 CREATE TABLE CoreRemovedTracks (
@@ -413,17 +589,21 @@ namespace Banshee.Database
                     ItemID              INTEGER
                 )
             ");
+
             // This index slows down queries were we shove data into the CoreCache.
             // Since we do that frequently, not using it.
             //Execute("CREATE INDEX CoreCacheModelId      ON CoreCache(ModelID)");
         }
         
+#endregion
+
+#region Legacy database migration
+        
         private void MigrateFromLegacyBanshee()
         {
-            Thread.Sleep (3000);
             Execute(@"
                 INSERT INTO CoreArtists 
-                    SELECT DISTINCT null, 0, null, Artist, 0 
+                    SELECT DISTINCT null, 0, null, Artist, NULL, 0 
                         FROM Tracks 
                         ORDER BY Artist
             ");
@@ -435,12 +615,12 @@ namespace Banshee.Database
                             FROM CoreArtists 
                             WHERE Name = Tracks.Artist
                             LIMIT 1),
-                        0, null, AlbumTitle, ReleaseDate, 0, 0, 0
+                        0, null, AlbumTitle, NULL, ReleaseDate, 0, 0, 0
                         FROM Tracks
                         ORDER BY AlbumTitle
             ");
             
-            Execute(@"
+            Execute (String.Format (@"
                 INSERT INTO CoreTracks
                     SELECT 
                         1,
@@ -454,12 +634,15 @@ namespace Banshee.Database
                                 AND a.ArtistID = b.ArtistID
                                 AND b.Name = Artist),
                         0,
-                        null,
+                        0,
+                        0,
                         Uri,
                         0,
                         MimeType,
                         0,
-                        Title,
+                        {0},
+                        {1},
+                        Title, NULL,
                         TrackNumber,
                         TrackCount,
                         0,
@@ -471,10 +654,11 @@ namespace Banshee.Database
                         NumberOfPlays,
                         0,
                         LastPlayedStamp,
+                        NULL,
                         DateAddedStamp,
                         DateAddedStamp
                         FROM Tracks
-            ");
+            ", (int)TrackMediaAttributes.Default, (int)StreamPlaybackError.None));
 
             Execute ("update coretracks set lastplayedstamp = NULL where lastplayedstamp = -62135575200");
 
@@ -493,14 +677,19 @@ namespace Banshee.Database
                     SELECT * FROM SmartPlaylists
             ");
 
-            ServiceManager.ServiceStarted += OnServiceStarted;
+            Execute ("UPDATE CoreSmartPlaylists SET PrimarySourceID = 1");
+            Execute ("UPDATE CorePlaylists SET PrimarySourceID = 1");
         }
 
+#endregion
+
+#region Utilities / Source / Service Stuff
+
         private void OnServiceStarted (ServiceStartedArgs args)
         {
             if (args.Service is UserJobManager) {
                 ServiceManager.ServiceStarted -= OnServiceStarted;
-                if (ServiceManager.SourceManager.Library != null) {
+                if (ServiceManager.SourceManager.MusicLibrary != null) {
                     RefreshMetadataDelayed ();
                 } else {
                     ServiceManager.SourceManager.SourceAdded += OnSourceAdded;
@@ -510,11 +699,25 @@ namespace Banshee.Database
 
         private void OnSourceAdded (SourceAddedArgs args)
         {
-            if (args.Source is Banshee.Library.LibrarySource) {
+            if (ServiceManager.SourceManager.MusicLibrary != null && ServiceManager.SourceManager.VideoLibrary != null) {
                 ServiceManager.SourceManager.SourceAdded -= OnSourceAdded;
                 RefreshMetadataDelayed ();
             }
         }
+        
+        private void ReloadAllSources (Client client)
+        {
+            Application.ClientStarted -= ReloadAllSources;
+            foreach (Source source in ServiceManager.SourceManager.Sources) {
+                if (source is ITrackModelSource) {
+                    ((ITrackModelSource)source).Reload ();
+                }
+            }
+        }
+        
+#endregion
+        
+#region Metadata Refresh Driver
 
         private void RefreshMetadataDelayed ()
         {
@@ -529,7 +732,7 @@ namespace Banshee.Database
 
         private void RefreshMetadataThread (object state)
         {
-            int total = ServiceManager.DbConnection.Query<int> ("SELECT count(*) FROM CoreTracks WHERE PrimarySourceID = 1");
+            int total = ServiceManager.DbConnection.Query<int> ("SELECT count(*) FROM CoreTracks");
 
             if (total <= 0) {
                 return;
@@ -542,7 +745,7 @@ namespace Banshee.Database
 
             HyenaSqliteCommand select_command = new HyenaSqliteCommand (
                 String.Format (
-                    "SELECT {0} FROM {1} WHERE {2} AND CoreTracks.PrimarySourceID = 1",
+                    "SELECT {0} FROM {1} WHERE {2}",
                     DatabaseTrackInfo.Provider.Select,
                     DatabaseTrackInfo.Provider.From,
                     DatabaseTrackInfo.Provider.Where
@@ -554,23 +757,40 @@ namespace Banshee.Database
                 while (reader.Read ()) {
                     DatabaseTrackInfo track = null;
                     try {
-                        track = DatabaseTrackInfo.Provider.Load (reader, 0);
-                        TagLib.File file = StreamTagger.ProcessUri (track.Uri);
-                        StreamTagger.TrackInfoMerge (track, file, true);
-                        track.Save (false);
+                        track = DatabaseTrackInfo.Provider.Load (reader);
 
-                        job.Status = String.Format ("{0} - {1}", track.DisplayArtistName, track.DisplayTrackTitle);
+                        if (!track.Uri.IsFile)
+                            continue;
+                        
+                        try {
+                            TagLib.File file = StreamTagger.ProcessUri (track.Uri);
+                            StreamTagger.TrackInfoMerge (track, file, true);
+                        } catch (Exception e) {
+                            Log.Warning (String.Format ("Failed to update metadata for {0}", track),
+                                e.GetType ().ToString (), false);
+                        }
+                        
+                        track.Save (false);
+                        track.Artist.Save ();
+                        track.Album.Save ();
                     } catch (Exception e) {
-                        Log.Warning (String.Format ("Failed to update metadata for {0}", track),
-                            e.GetType ().ToString (), false);
+                        Log.Warning (String.Format ("Failed to update metadata for {0}", track), e.ToString (), false);
+                        throw;
                     }
 
+                    job.Status = String.Format ("{0} - {1}", track.DisplayArtistName, track.DisplayTrackTitle);
                     job.Progress = (double)++count / (double)total;
                 }
             }
 
+            if (ServiceManager.DbConnection.Query<int> ("SELECT count(*) FROM CoreConfiguration WHERE Key = 'MetadataVersion'") == 0) {
+                Execute (String.Format ("INSERT INTO CoreConfiguration VALUES (null, 'MetadataVersion', {0})", CURRENT_METADATA_VERSION));
+            } else {
+                Execute (String.Format ("UPDATE CoreConfiguration SET Value = {0} WHERE Key = 'MetadataVersion'", CURRENT_METADATA_VERSION));
+            }
+
             job.Finish ();
-            ServiceManager.SourceManager.Library.NotifyTracksChanged ();
+            ServiceManager.SourceManager.MusicLibrary.NotifyTracksChanged ();
         }
         
 #endregion
