@@ -94,8 +94,6 @@ namespace Banshee.Sources
             DELETE FROM CoreTracks WHERE PrimarySourceId = ?
         ");
 
-        public readonly SchemaEntry<bool> ExpandedSchema;
-
         private int dbid;
         public int DbId {
             get {
@@ -118,12 +116,6 @@ namespace Banshee.Sources
             }
         }
 
-        private bool supports_playlists = true;
-        public bool SupportsPlaylists {
-            get { return supports_playlists; }
-            protected set { supports_playlists = value; }
-        }
-
         public ErrorSource ErrorSource {
             get {
                 if (error_source == null) {
@@ -135,13 +127,6 @@ namespace Banshee.Sources
             }
         }
 
-
-        private bool is_local = false;
-        public bool IsLocal {
-            get { return is_local; }
-            protected set { is_local = value; }
-        }
-
         public delegate void TrackEventHandler (Source sender, TrackEventArgs args);
 
         public event TrackEventHandler TracksAdded;
@@ -178,29 +163,12 @@ namespace Banshee.Sources
         protected PrimarySource (string generic_name, string name, string id, int order) : base (generic_name, name, id, order)
         {
             PrimarySourceInitialize ();
-
-            ExpandedSchema = new SchemaEntry<bool> (
-                String.Format ("sources.{0}", ConfigurationId), "expanded", true, "Is source expanded", "Is source expanded"
-            );
         }
 
         protected PrimarySource () : base ()
         {
-            ExpandedSchema = new SchemaEntry<bool> (
-                String.Format ("sources.{0}", ConfigurationId), "expanded", true, "Is source expanded", "Is source expanded"
-            );
         }
 
-        public override bool? AutoExpand {
-            get { return ExpandedSchema.Get (); }
-        }
-
-        public override bool Expanded {
-            get { return ExpandedSchema.Get (); }
-            set { ExpandedSchema.Set (value); }
-        }
-
-
         public virtual void Dispose ()
         {
             if (Application.ShuttingDown)
@@ -272,11 +240,6 @@ namespace Banshee.Sources
             );
         }
 
-        public virtual void CopyTrackTo (DatabaseTrackInfo track, SafeUri uri, BatchUserJob job)
-        {
-            Log.WarningFormat ("CopyToTrack not implemented for source {0}", this);
-        }
-
         internal void NotifyTracksAdded ()
         {
             OnTracksAdded ();
@@ -355,13 +318,7 @@ namespace Banshee.Sources
         protected override void OnTracksChanged (params QueryField [] fields)
         {
             ThreadAssist.SpawnFromMain (delegate {
-                if (NeedsReloadWhenFieldsChanged (fields)) {
-                    Reload ();
-                } else {
-                    InvalidateCaches ();
-                }
-
-                System.Threading.Thread.Sleep (150);
+                Reload ();
 
                 TrackEventHandler handler = TracksChanged;
                 if (handler != null) {
@@ -472,9 +429,7 @@ namespace Banshee.Sources
 
         public override bool AcceptsInputFromSource (Source source)
         {
-            return base.AcceptsInputFromSource (source) && source.Parent != this
-                && (source.Parent is PrimarySource || source is PrimarySource)
-                && !(source.Parent is Banshee.Library.LibrarySource);
+            return base.AcceptsInputFromSource (source) && source.Parent != this;
         }
 
         public override bool AddSelectedTracks (Source source)
