@@ -35,11 +35,13 @@ using System.Collections.ObjectModel;
 
 using Mono.Unix;
 
+using Hyena;
 using Hyena.Data;
 using Hyena.Query;
 
 using Banshee.Base;
 using Banshee.Collection;
+using Banshee.Configuration;
 using Banshee.ServiceStack;
 
 namespace Banshee.Sources
@@ -48,6 +50,7 @@ namespace Banshee.Sources
     {
         private Source parent;
         private PropertyStore properties = new PropertyStore ();
+        protected SourceMessage status_message;
         private List<SourceMessage> messages = new List<SourceMessage> ();
         private List<Source> child_sources = new List<Source> ();
         private ReadOnlyCollection<Source> read_only_children;
@@ -57,6 +60,8 @@ namespace Banshee.Sources
         public event EventHandler MessageNotify;
         public event SourceEventHandler ChildSourceAdded;
         public event SourceEventHandler ChildSourceRemoved;
+
+        public delegate void OpenPropertiesDelegate ();
         
         protected Source (string generic_name, string name, int order)
         {
@@ -64,6 +69,22 @@ namespace Banshee.Sources
             Name = name;
             Order = order;
 
+            SourceInitialize ();
+        }
+
+        protected Source ()
+        {
+        }
+
+        // This method is chained to subclasses intialize methods,
+        // allowing at any state for delayed intialization by using the empty ctor.
+        protected virtual void Initialize ()
+        {
+            SourceInitialize ();
+        }
+
+        private void SourceInitialize ()
+        {
             // If this source is not defined in Banshee.Services, set its
             // ResourceAssembly to the assembly where it is defined.
             Assembly asm = Assembly.GetAssembly (this.GetType ());//Assembly.GetCallingAssembly ();
@@ -86,11 +107,14 @@ namespace Banshee.Sources
                 tm_source.TrackModel.Parent = this;
                 ServiceManager.DBusServiceManager.RegisterObject (tm_source.TrackModel);
                 
-                tm_source.ArtistModel.Parent = this;
-                ServiceManager.DBusServiceManager.RegisterObject (tm_source.ArtistModel);
-                
-                tm_source.AlbumModel.Parent = this;
-                ServiceManager.DBusServiceManager.RegisterObject (tm_source.AlbumModel);
+                // TODO if/when browsable models can be added/removed on the fly, this would need to change to reflect that
+                foreach (IListModel model in tm_source.FilterModels) {
+                    Banshee.Collection.ExportableModel exportable = model as Banshee.Collection.ExportableModel;
+                    if (exportable != null) {
+                        exportable.Parent = this;
+                        ServiceManager.DBusServiceManager.RegisterObject (exportable);
+                    }
+                }
             }
         }
 
@@ -142,9 +166,11 @@ namespace Banshee.Sources
         public virtual void AddChildSource (Source child)
         {
             lock (Children) {
-                child.SetParentSource (this);
-                child_sources.Add (child);
-                OnChildSourceAdded (child);
+                if (!child_sources.Contains (child)) {
+                    child.SetParentSource (this);
+                    child_sources.Add (child);
+                    OnChildSourceAdded (child);
+                }
             }
         }
 
@@ -158,7 +184,7 @@ namespace Banshee.Sources
                 child_sources.Remove (child);
                 
                 if (ServiceManager.SourceManager.ActiveSource == child) {
-                    ServiceManager.SourceManager.SetActiveSource (ServiceManager.SourceManager.DefaultSource);
+                    ServiceManager.SourceManager.SetActiveSource (this);
                 }
                 
                 OnChildSourceRemoved (child);
@@ -208,11 +234,49 @@ namespace Banshee.Sources
 #endregion
         
 #region Protected Methods
+        
+        public virtual void SetStatus (string message, bool error)
+        {
+            SetStatus (message, !error, !error, error ? "dialog-error" : null);
+        }
+
+        public virtual void SetStatus (string message, bool can_close, bool is_spinning, string icon_name)
+        {
+            lock (this) {
+                if (status_message == null) {
+                    status_message = new SourceMessage (this);
+                    PushMessage (status_message);
+                }
+            
+                string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
+                
+                status_message.FreezeNotify ();
+                status_message.Text = String.Format (GLib.Markup.EscapeText (message), status_name);
+                status_message.CanClose = can_close;
+                status_message.IsSpinning = is_spinning;
+                status_message.SetIconName (icon_name);
+                status_message.ClearActions ();
+            }
+                
+            status_message.ThawNotify ();
+        }
+
+        public virtual void HideStatus ()
+        {
+            lock (this) {
+                if (status_message != null) {
+                    RemoveMessage (status_message);
+                    status_message = null;
+                }
+            }
+        }
+
 
         protected virtual void PushMessage (SourceMessage message)
         {
             lock (this) {
                 messages.Insert (0, message);
+                message.Updated += HandleMessageUpdated;
             }
             
             OnMessageNotify ();
@@ -224,6 +288,7 @@ namespace Banshee.Sources
                 lock (this) {
                     if (messages.Count > 0) {
                         SourceMessage message = messages[0];
+                        message.Updated -= HandleMessageUpdated;
                         messages.RemoveAt (0);
                         return message;
                     }
@@ -235,6 +300,19 @@ namespace Banshee.Sources
             }
         }
         
+        protected virtual void ClearMessages ()
+        {
+            lock (this) {
+                if (messages.Count > 0) {
+                    foreach (SourceMessage message in messages) {
+                        message.Updated -= HandleMessageUpdated;
+                    }
+                    messages.Clear ();
+                    OnMessageNotify ();
+                }
+            }
+        }
+        
         private void TestMessages ()
         {
             int count = 0;
@@ -277,10 +355,16 @@ namespace Banshee.Sources
         {
             lock (this) {
                 if (messages.Remove (message)) {
+                    message.Updated -= HandleMessageUpdated;
                     OnMessageNotify ();
                 }
             }   
         }
+
+        private void HandleMessageUpdated (object o, EventArgs args)
+        {
+            OnMessageNotify ();
+        }
         
         protected virtual void OnMessageNotify ()
         {
@@ -317,12 +401,19 @@ namespace Banshee.Sources
                 handler (this, EventArgs.Empty);
             }
         }
-        
-        protected virtual void OnUserNotifyUpdated ()
+
+        public void NotifyUser ()
         {
-            EventHandler handler = UserNotifyUpdated;
-            if (handler != null) {
-                handler (this, EventArgs.Empty);
+            OnUserNotifyUpdated ();
+        }
+
+        protected void OnUserNotifyUpdated ()
+        {
+            if (this != ServiceManager.SourceManager.ActiveSource) {
+                EventHandler handler = UserNotifyUpdated;
+                if (handler != null) {
+                    handler (this, EventArgs.Empty);
+                }
             }
         }
         
@@ -350,21 +441,28 @@ namespace Banshee.Sources
         public Source Parent {
             get { return parent; }
         }
+        
+        private string unique_id;
+        public string UniqueId {
+            get { return unique_id ?? unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId); }
+        }
+        
+        protected abstract string TypeUniqueId { get; }
 
         public virtual bool CanRename {
-            get { return true; }
+            get { return false; }
         }
 
         public virtual bool HasProperties {
             get { return false; }
         }
 
-        public string Name {
+        public virtual string Name {
             get { return properties.GetString ("Name"); }
             set { properties.SetString ("Name", value); }
         }
 
-        public string GenericName {
+        public virtual string GenericName {
             get { return properties.GetString ("GenericName"); }
             set { properties.SetString ("GenericName", value); }
         }
@@ -383,7 +481,7 @@ namespace Banshee.Sources
         }
         
         public virtual bool CanSearch {
-            get { return true; }
+            get { return false; }
         }
                 
         public virtual string FilterQuery {
@@ -413,8 +511,47 @@ namespace Banshee.Sources
             get { return true; }
         }
         
-        public abstract int Count { get; }
+        public virtual int Count {
+            get { return 0; }
+        }
+        
+        public string ConfigurationId {
+            get { return (Parent == null ? UniqueId : Parent.UniqueId).Replace ('.', '_'); }
+        }
+
         public virtual int FilteredCount { get { return Count; } }
+                
+        public virtual string TrackModelPath {
+            get { return null; }
+        }
+        
+#endregion
+
+#region Status Message Stuff        
+        
+        private static DurationStatusFormatters duration_status_formatters = new DurationStatusFormatters ();
+        public static DurationStatusFormatters DurationStatusFormatters {
+            get { return duration_status_formatters; }
+        }
+        
+        protected virtual int StatusFormatsCount {
+            get { return duration_status_formatters.Count; }
+        }
+        
+        protected virtual int CurrentStatusFormat {
+            get { return ConfigurationClient.Get<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", 0); }
+            set { ConfigurationClient.Set<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", value); }
+        }
+        
+        public void CycleStatusFormat ()
+        {
+            int new_status_format = CurrentStatusFormat + 1;
+            if (new_status_format >= StatusFormatsCount) {
+                new_status_format = 0;
+            }
+            
+            CurrentStatusFormat = new_status_format;
+        }
 
         public virtual string GetStatusText ()
         {
@@ -428,23 +565,9 @@ namespace Banshee.Sources
             
             builder.AppendFormat (Catalog.GetPluralString ("{0} item", "{0} items", count), count);
             
-            if (this is IDurationAggregator) {
+            if (this is IDurationAggregator && StatusFormatsCount > 0) {
                 builder.Append (", ");
-
-                TimeSpan span = (this as IDurationAggregator).Duration; 
-                if (span.Days > 0) {
-                    double days = span.Days + (span.Hours / 24.0);
-                    builder.AppendFormat (Catalog.GetPluralString ("{0} day", "{0:0.0} days", 
-                        (int)Math.Ceiling (days)), days);
-                } else if (span.Hours > 0) {
-                    double hours = span.Hours + (span.Minutes / 60.0);
-                    builder.AppendFormat (Catalog.GetPluralString ("{0} hour", "{0:0.0} hours", 
-                        (int)Math.Ceiling (hours)), hours);
-                } else {
-                    double minutes = span.Minutes + (span.Seconds / 60.0);
-                    builder.AppendFormat (Catalog.GetPluralString ("{0} minute", "{0:0.0} minutes", 
-                        (int)Math.Ceiling (minutes)), minutes);
-                }
+                duration_status_formatters[CurrentStatusFormat] (builder, ((IDurationAggregator)this).Duration);
             }
 
             if (this is IFileSizeAggregator) {
@@ -458,8 +581,15 @@ namespace Banshee.Sources
             return builder.ToString ();
         }
         
+#endregion
+
+        public override string ToString ()
+        {
+            return Name;
+        }
+        
         string IService.ServiceName {
-            get { return DBusServiceManager.MakeDBusSafeString (Name) + "Source"; }
+            get { return String.Format ("{0}{1}", DBusServiceManager.MakeDBusSafeString (Name), "Source"); }
         }
         
         IDBusExportable IDBusExportable.Parent {
@@ -471,12 +601,5 @@ namespace Banshee.Sources
                 }
             }
         }
-        
-        public virtual string TrackModelPath {
-            get { return null; }
-        }
-        
-#endregion
-        
     }
 }
