@@ -86,7 +86,6 @@ namespace Banshee.Podcasting
             feeds_manager.FeedManager.FeedsChanged += OnFeedsChanged;
 
             ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, PlayerEvent.StateChange);
-            ServiceManager.Get<DBusCommandService> ().ArgumentPushed += OnCommandLineArgument;
 
             InitializeInterface ();
         }
@@ -99,7 +98,7 @@ namespace Banshee.Podcasting
                     Hyena.Log.Information ("Migrating Podcast Feeds and Items");
                     ServiceManager.DbConnection.Execute(@"
                         INSERT INTO PodcastSyndications (FeedID, Title, Url, Link,
-                            Description, ImageUrl, LastBuildDate, AutoDownload, IsSubscribed)
+                            Description, ImageUrl, LastBuildDate, SyncSetting, IsSubscribed)
                             SELECT 
                                 PodcastFeedID,
                                 Title,
@@ -172,21 +171,6 @@ namespace Banshee.Podcasting
                 DatabaseConfigurationClient.Client.Set<int> ("Podcast", "Version", 1);
             }
 
-            if (DatabaseConfigurationClient.Client.Get<int> ("Podcast", "Version", 0) < 3) {
-                // We were using the Link as the fallback if the actual Guid was missing, but that was a poor choice
-                // since it is not always unique.  We now use the title and pubdate combined.
-                ServiceManager.DbConnection.Execute ("UPDATE PodcastItems SET Guid = NULL");
-                foreach (FeedItem item in FeedItem.Provider.FetchAll ()) {
-                    item.Guid = null;
-                    if (item.Feed == null || FeedItem.Exists (item.Feed.DbId, item.Guid)) {
-                        item.Delete (false);
-                    } else {
-                        item.Save ();
-                    }
-                }
-
-                DatabaseConfigurationClient.Client.Set<int> ("Podcast", "Version", 3);
-            }
         }
         
         public void Initialize ()
@@ -218,7 +202,6 @@ namespace Banshee.Podcasting
             }
             
             ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
-            ServiceManager.Get<DBusCommandService> ().ArgumentPushed -= OnCommandLineArgument;
 
             if (download_manager_iface != null) {
                 download_manager_iface.Dispose ();                
@@ -243,23 +226,6 @@ namespace Banshee.Podcasting
             }
         }
         
-        private void OnCommandLineArgument (string uri, object value, bool isFile)
-        {
-            if (!isFile || String.IsNullOrEmpty (uri)) {
-                return;
-            }
-            
-            // Handle podcast URIs
-            if (uri.StartsWith ("feed://")) {
-                string url = String.Format ("http://{0}", uri.Substring (7));
-                Log.DebugFormat ("Subscribing to podcast at {0}", url);
-
-                // TODO replace autodownload w/ actual default preference
-                FeedsManager.Instance.FeedManager.CreateFeed (url, FeedAutoDownload.None);
-                source.NotifyUser ();
-            }
-        }
-        
         private void RefreshArtworkFor (Feed feed)
         {
             if (feed.LastDownloadTime != DateTime.MinValue)
@@ -287,13 +253,8 @@ namespace Banshee.Podcasting
             }
         }
         
-        internal static bool IgnoreItemChanges = false;
-        
         private void OnItemChanged (FeedItem item)
         {
-            if (IgnoreItemChanges)
-                return;
-
             PodcastTrackInfo track = PodcastTrackInfo.GetByItemId (item.DbId);
             if (track != null) {
                 track.SyncWithFeedItem ();
@@ -361,7 +322,6 @@ namespace Banshee.Podcasting
                 PodcastTrackInfo pi = new PodcastTrackInfo (item);
                 pi.PrimarySource = source;
                 pi.Save (true);
-                source.NotifyUser ();
             } else {
                 item.Delete (false);                      
             }
