@@ -40,7 +40,7 @@ namespace Hyena.Data.Gui
 {
     public delegate int ListViewRowHeightHandler (Widget widget);
 
-    public partial class ListView<T> : ListViewBase
+    public partial class ListView<T> : Widget
     {
         private Cairo.Context cairo_context;
         private CellContext cell_context;
@@ -58,14 +58,9 @@ namespace Hyena.Data.Gui
             base.OnStyleSet (old_style);
             RecomputeRowHeight = true;
             theme = new GtkTheme (this);
-
-            // Save the drawable so we can reuse it
-            Gdk.Drawable drawable = cell_context != null ? cell_context.Drawable : null;
-
             cell_context = new CellContext ();
             cell_context.Theme = theme;
             cell_context.Widget = this;
-            cell_context.Drawable = drawable;
         }
          
         protected override bool OnExposeEvent (EventExpose evnt)
@@ -314,7 +309,6 @@ namespace Hyena.Data.Gui
         {
             ColumnCell cell = column_cache[column_index].Column.GetCell (0);
             cell.BindListItem (item);
-            ColumnCellDataProvider (cell, item);
             
             if (dragging) {
                 Cairo.Color fill_color = Theme.Colors.GetWidgetColor (GtkColorClass.Base, StateType.Normal);
@@ -378,10 +372,6 @@ namespace Hyena.Data.Gui
             }
         }
         
-        protected virtual void ColumnCellDataProvider (ColumnCell cell, object boundItem)
-        {
-        }
-        
         private bool rules_hint = false;
         public bool RulesHint {
             get { return rules_hint; }
