@@ -35,9 +35,12 @@ namespace Hyena.Query
 {
     public class QueryField : IAliasedObject
     {
-        private Type value_type;
-        public Type ValueType {
-            get { return value_type; }
+        private bool no_custom_format;
+        private bool column_lowered;
+
+        private Type [] value_types;
+        public Type [] ValueTypes {
+            get { return value_types; }
         }
 
         private string name;
@@ -77,30 +80,47 @@ namespace Hyena.Query
         }
 
         public QueryField (string name, string label, string column, bool isDefault, params string [] aliases)
-            : this (name, label, column, typeof(StringQueryValue), isDefault, aliases)
+            : this (name, label, column, new Type [] {typeof(StringQueryValue)}, isDefault, aliases)
         {
         }
 
         public QueryField (string name, string label, string column, Type valueType, params string [] aliases)
-            : this (name, label, column, valueType, false, aliases)
+            : this (name, label, column, new Type [] {valueType}, false, aliases)
         {
         }
 
-        public QueryField (string name, string label, string column, Type valueType, bool isDefault, params string [] aliases)
+        public QueryField (string name, string label, string column, Type [] valueTypes, params string [] aliases)
+            : this (name, label, column, valueTypes, false, aliases)
+        {
+        }
+
+        public QueryField (string name, string label, string column, Type [] valueTypes, bool isDefault, params string [] aliases)
         {
             this.name = name;
             this.label = label;
             this.column = column;
-            this.value_type = valueType;
+            this.value_types = valueTypes;
             this.is_default = isDefault;
             this.aliases = aliases;
 
-            QueryValue.AddValueType (value_type);
+            this.no_custom_format = (Column.IndexOf ("{0}") == -1 && Column.IndexOf ("{1}") == -1);
+            this.column_lowered = (Column.IndexOf ("Lowered") != -1);
+
+            if (!no_custom_format) {
+                // Ensure we have parens around any custom 'columns' that may be an OR of two columns
+                this.column = String.Format ("({0})", this.column);
+            }
+
+            foreach (Type value_type in valueTypes) {
+                QueryValue.AddValueType (value_type);
+            }
         }
 
-        public QueryValue CreateQueryValue ()
+        public IEnumerable<QueryValue> CreateQueryValues ()
         {
-            return Activator.CreateInstance (ValueType) as QueryValue;
+            foreach (Type type in ValueTypes) {
+                yield return Activator.CreateInstance (type) as QueryValue;
+            }
         }
 
         public string ToTermString (string op, string value)
@@ -111,23 +131,42 @@ namespace Hyena.Query
         public string ToSql (Operator op, QueryValue qv)
         {
             string value = qv.ToSql ();
+
             if (op == null) op = qv.OperatorSet.First;
-            if (Column.IndexOf ("{0}") == -1 && Column.IndexOf ("{1}") == -1) {
-                if (ValueType == typeof(StringQueryValue)) {
-                    // Match string values literally and against a lower'd version 
-                    return String.Format ("({0} {1} OR LOWER({0}) {2})", Column,
-                        String.Format (op.SqlFormat, value),
-                        String.Format (op.SqlFormat, value.ToLower ())
-                    );
+
+            StringBuilder sb = new StringBuilder ();
+
+            if (no_custom_format) {
+                if (qv is StringQueryValue) {
+                    if (column_lowered) {
+                        // The column is pre-lowered, only no need to call lower() in SQL
+                        sb.AppendFormat ("{0} {1}", Column, String.Format (op.SqlFormat, value.ToLower ()));
+                    } else {
+                        // Match string values literally and against a lower'd version.  Mostly a workaround
+                        // the fact that Sqlite's lower() method only works for ASCII (meaning even with this,
+                        // we're not getting 100% case-insensitive matching).
+                        sb.AppendFormat ("({0} {1} OR LOWER({0}) {2})", Column,
+                            String.Format (op.SqlFormat, value),
+                            String.Format (op.SqlFormat, value.ToLower ())
+                        );
+                    }
                 } else {
-                    return String.Format ("{0} {1}", Column, String.Format (op.SqlFormat, value));
+                    sb.AppendFormat ("{0} {1}", Column, String.Format (op.SqlFormat, value));
+                }
+
+                if (op.IsNot) {
+                    return String.Format ("({0} OR {1} IS NULL)", sb.ToString (), Column);
+                } else {
+                    return String.Format ("({0} AND {1} IS NOT NULL)", sb.ToString (), Column);
                 }
             } else {
-                return String.Format (
+                sb.AppendFormat (
                     Column, String.Format (op.SqlFormat, value),
                     value, op.IsNot ? "NOT" : null
                 );
             }
+
+            return sb.ToString ();
         }
 
         public static string ToTermString (string alias, string op, string value)
